/*

Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.

*/
module derelict.sdl.ttf;

private
{
    import derelict.util.loader;
    import derelict.util.exception;
    import derelict.util.compat;
    import derelict.sdl.sdl;

}

enum : Uint8
{
    SDL_TTF_MAJOR_VERSION = 2,
    SDL_TTF_MINOR_VERSION = 0,
    SDL_TTF_PATCHLEVEL    = 10,
}
alias SDL_TTF_MAJOR_VERSION TTF_MAJOR_VERSION;
alias SDL_TTF_MINOR_VERSION TTF_MINOR_VERSION;
alias SDL_TTF_PATCHLEVEL TTF_PATCHLEVEL;

enum
{
    UNICODE_BOM_NATIVE = 0xFEFF,
    UNICODE_BOM_SWAPPED = 0xFFFE,
    TTF_STYLE_NORMAL = 0x00,
    TTF_STYLE_BOLD = 0x01,
    TTF_STYLE_ITALIC = 0x02,
    TTF_STYLE_UNDERLINE = 0x04,
    TTF_STYLE_STRIKETHROUGH = 0x08,
}

enum
{
    TTF_HINTING_NORMAL = 0,
    TTF_HINTING_LIGHT = 1,
    TTF_HINTING_MONO = 2,
    TTF_HINTING_NONE = 3,
}

alias SDL_SetError TTF_SetError;
alias SDL_GetError TTF_GetError;

struct _TTF_Font {}
alias _TTF_Font TTF_Font;

void SDL_TTF_VERSION(SDL_version* X)
{
    X.major = SDL_TTF_MAJOR_VERSION;
    X.minor = SDL_TTF_MINOR_VERSION;
    X.patch = SDL_TTF_PATCHLEVEL;
}

void TTF_VERSION(SDL_version* X) { SDL_TTF_VERSION(X); }


extern (C)
{
    mixin(gsharedString!() ~
    "
    CSDLVERPTR function() TTF_Linked_Version;
    void function(int) TTF_ByteSwappedUNICODE;
    int function() TTF_Init;
    TTF_Font * function (in char*, int) TTF_OpenFont;
    TTF_Font * function (in char*, int, long ) TTF_OpenFontIndex;
    TTF_Font * function (SDL_RWops*, int, int) TTF_OpenFontRW;
    TTF_Font * function (SDL_RWops*, int, int, long) TTF_OpenFontIndexRW;
    int function (in TTF_Font*) TTF_GetFontStyle;
    void function (in TTF_Font*, int style) TTF_SetFontStyle;
    int function(in TTF_Font*) TTF_GetFontOutline;
    void function(TTF_Font*, int) TTF_SetFontOutline;
    int function(in TTF_Font*) TTF_GetFontHinting;
    void function(TTF_Font*, int) TTF_SetFontHinting;
    int function(in TTF_Font*) TTF_FontHeight;
    int function(in TTF_Font*) TTF_FontAscent;
    int function(in TTF_Font*) TTF_FontDescent;
    int function(in TTF_Font*) TTF_FontLineSkip;
    int function(in TTF_Font*) TTF_GetFontKerning;
    void function(TTF_Font*, int) TTF_SetFontKerning;
    int function(in TTF_Font*) TTF_FontFaces;
    int function(in TTF_Font*) TTF_FontFaceIsFixedWidth;
    char* function(in TTF_Font*) TTF_FontFaceFamilyName;
    char* function(in TTF_Font*) TTF_FontFaceStyleName;
    int function(in TTF_Font*, Uint16) TTF_GlyphIsProvided;
    int function (TTF_Font*, Uint16, int*, int*, int*, int*, int*) TTF_GlyphMetrics;
    int function (TTF_Font*, in char*, int*, int*) TTF_SizeText;
    int function (TTF_Font*, in char*, int*, int*) TTF_SizeUTF8;
    int function (TTF_Font*, Uint16*, int*, int*) TTF_SizeUNICODE;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color) TTF_RenderText_Solid;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color) TTF_RenderUTF8_Solid;
    SDL_Surface* function (TTF_Font*, in Uint16*, SDL_Color) TTF_RenderUNICODE_Solid;
    SDL_Surface* function (TTF_Font*, Uint16, SDL_Color) TTF_RenderGlyph_Solid;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color, SDL_Color) TTF_RenderText_Shaded;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color, SDL_Color) TTF_RenderUTF8_Shaded;
    SDL_Surface* function (TTF_Font*, in Uint16*, SDL_Color, SDL_Color) TTF_RenderUNICODE_Shaded;
    SDL_Surface* function (TTF_Font*, Uint16, SDL_Color, SDL_Color) TTF_RenderGlyph_Shaded;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color) TTF_RenderText_Blended;
    SDL_Surface* function (TTF_Font*, in char*, SDL_Color) TTF_RenderUTF8_Blended;
    SDL_Surface* function (TTF_Font*, in Uint16*, SDL_Color) TTF_RenderUNICODE_Blended;
    SDL_Surface* function (TTF_Font*, Uint16, SDL_Color) TTF_RenderGlyph_Blended;
    void function (TTF_Font*) TTF_CloseFont;
    void function () TTF_Quit;
    int function () TTF_WasInit;
    ");

    alias TTF_RenderText_Shaded TTF_RenderText;
    alias TTF_RenderUTF8_Shaded TTF_RenderUTF8;
    alias TTF_RenderUNICODE_Shaded TTF_RenderUNICODE;
}

class DerelictSDLttfLoader : SharedLibLoader
{
public:
    this()
    {
        super(
            "SDL_ttf.dll",
            "libSDL_ttf.so, libSDL_ttf-2.0.so, libSDL_ttf-2.0.so.0",
            "./Frameworks/SDL_ttf.framework/SDL_ttf, /Library/Frameworks/SDL_ttf.framework/SDL_ttf, /System/Library/Frameworks/SDL_ttf.framework/SDL_ttf"
        );
    }

protected:
    override void loadSymbols()
    {
        if(!DerelictSDL.isLoaded)
        {
            throw new SharedLibLoadException("DerelictSDL must be loaded before attempting to load DerelictSDLttf!");
        }

        bindFunc(cast(void**)&TTF_Linked_Version, "TTF_Linked_Version");
        bindFunc(cast(void**)&TTF_ByteSwappedUNICODE, "TTF_ByteSwappedUNICODE");
        bindFunc(cast(void**)&TTF_Init, "TTF_Init");
        bindFunc(cast(void**)&TTF_OpenFont, "TTF_OpenFont");
        bindFunc(cast(void**)&TTF_OpenFontIndex, "TTF_OpenFontIndex");
        bindFunc(cast(void**)&TTF_OpenFontRW, "TTF_OpenFontRW");
        bindFunc(cast(void**)&TTF_OpenFontIndexRW, "TTF_OpenFontIndexRW");
        bindFunc(cast(void**)&TTF_GetFontStyle, "TTF_GetFontStyle");
        bindFunc(cast(void**)&TTF_SetFontStyle, "TTF_SetFontStyle");
        bindFunc(cast(void**)&TTF_GetFontOutline, "TTF_GetFontOutline");
        bindFunc(cast(void**)&TTF_SetFontOutline, "TTF_SetFontOutline");
        bindFunc(cast(void**)&TTF_GetFontHinting, "TTF_GetFontHinting");
        bindFunc(cast(void**)&TTF_SetFontHinting, "TTF_SetFontHinting");
        bindFunc(cast(void**)&TTF_FontHeight, "TTF_FontHeight");
        bindFunc(cast(void**)&TTF_FontAscent, "TTF_FontAscent");
        bindFunc(cast(void**)&TTF_FontDescent, "TTF_FontDescent");
        bindFunc(cast(void**)&TTF_FontLineSkip, "TTF_FontLineSkip");
        bindFunc(cast(void**)&TTF_GetFontKerning, "TTF_GetFontKerning");
        bindFunc(cast(void**)&TTF_SetFontKerning, "TTF_SetFontKerning");
        bindFunc(cast(void**)&TTF_FontFaces, "TTF_FontFaces");
        bindFunc(cast(void**)&TTF_FontFaceIsFixedWidth, "TTF_FontFaceIsFixedWidth");
        bindFunc(cast(void**)&TTF_FontFaceFamilyName, "TTF_FontFaceFamilyName");
        bindFunc(cast(void**)&TTF_FontFaceStyleName, "TTF_FontFaceStyleName");
        bindFunc(cast(void**)&TTF_GlyphIsProvided, "TTF_GlyphIsProvided");
        bindFunc(cast(void**)&TTF_GlyphMetrics, "TTF_GlyphMetrics");
        bindFunc(cast(void**)&TTF_SizeText, "TTF_SizeText");
        bindFunc(cast(void**)&TTF_SizeUTF8, "TTF_SizeUTF8");
        bindFunc(cast(void**)&TTF_SizeUNICODE, "TTF_SizeUNICODE");
        bindFunc(cast(void**)&TTF_RenderText_Solid, "TTF_RenderText_Solid");
        bindFunc(cast(void**)&TTF_RenderUTF8_Solid, "TTF_RenderUTF8_Solid");
        bindFunc(cast(void**)&TTF_RenderUNICODE_Solid, "TTF_RenderUNICODE_Solid");
        bindFunc(cast(void**)&TTF_RenderGlyph_Solid, "TTF_RenderGlyph_Solid");
        bindFunc(cast(void**)&TTF_RenderText_Shaded, "TTF_RenderText_Shaded");
        bindFunc(cast(void**)&TTF_RenderUTF8_Shaded, "TTF_RenderUTF8_Shaded");
        bindFunc(cast(void**)&TTF_RenderUNICODE_Shaded, "TTF_RenderUNICODE_Shaded");
        bindFunc(cast(void**)&TTF_RenderGlyph_Shaded, "TTF_RenderGlyph_Shaded");
        bindFunc(cast(void**)&TTF_RenderText_Blended, "TTF_RenderText_Blended");
        bindFunc(cast(void**)&TTF_RenderUTF8_Blended, "TTF_RenderUTF8_Blended");
        bindFunc(cast(void**)&TTF_RenderUNICODE_Blended, "TTF_RenderUNICODE_Blended");
        bindFunc(cast(void**)&TTF_RenderGlyph_Blended, "TTF_RenderGlyph_Blended");
        bindFunc(cast(void**)&TTF_CloseFont, "TTF_CloseFont");
        bindFunc(cast(void**)&TTF_Quit, "TTF_Quit");
        bindFunc(cast(void**)&TTF_WasInit, "TTF_WasInit");
    }
}

DerelictSDLttfLoader DerelictSDLttf;

static this()
{
    DerelictSDLttf = new DerelictSDLttfLoader();
}

static ~this()
{
    DerelictSDLttf.unload();
}