module texpzl.guillotinemap;

import texpzl.texpzl;
import std.algorithm;
import std.math;
import std.stdio;


struct GuillotineRect {
    enum Type { invalid, singleRect, virtualPair }
    Type type;
    Rect r1, r2;

    void minimize() {
        final switch (type) {
        case Type.invalid:
            break;
        case Type.singleRect:
            if (r1.width == 0 || r1.height == 0)
                type = Type.invalid;
            break;
        case Type.virtualPair:
            if (r2.width == 0 || r2.height == 0) {
                r1 = r2;
                type = Type.singleRect;
                minimize();
            }
            break;
        }
    }
}


struct GuillotineMap {
static:


int doMapping(ref Images img, bool autoroll) {
        // 理論最小サイズ取得
        int min_pixels;
        foreach (ref a; img.assigns) {
            if (autoroll && a.height > a.width)
                a.roll ^= true;
            min_pixels += a.width * a.height;
        }
        
        img.assigns.sort;

        auto w = nearPow2(cast(int)ceil(sqrt(cast(real)min_pixels)));
        
        while (!procMapping(img, w, w, autoroll)) {
            w *= 2;
        }
        return w;
    }

    bool procMapping(ref Images img, int width, int height, bool autoroll) {
        int lineheight=0;
        int posx=0, posy=0;

        GuillotineRect[] rects;
        rects ~= GuillotineRect(GuillotineRect.Type.singleRect, Rect(0, 0, width, height));

        bool containable(Rect area, int w, int h) {
            return (area.width >= w) && (area.height >= h);
        }
        GuillotineRect guillotineCut(Rect r, int w, int h) {
            GuillotineRect cell;
            cell.type = GuillotineRect.Type.virtualPair;
            auto newr1 = Rect(r.x + w,
                              r.y,
                              r.width - w,
                              r.height);
            auto newr2 = Rect(r.x,
                              r.y + h,
                              r.width,
                              r.height - h);
            //writeln("cut! ", r, "\t", [w, h], "\t", newr1, "\t", newr2);
            cell.r1 = newr1;
            cell.r2 = newr2;
            return cell;
        }

        while (1) {
            int validationcounter=0;
            /+writeln("frame");
            foreach (r; rects)
                writeln(r);
            +/
            
            foreach (i; 0 .. rects.length) {
                auto guillotineCell() @property { return &rects[i]; }
                foreach (ref image; img.assigns) {
                    if (image.valid) // 使用済み
                        continue;

                    // cont_type 第一ビット：内包可能か　第二ビット：回転が必要　第三ビット：二番目の仮想Rectか
                    final switch (guillotineCell.type) {
                    case GuillotineRect.Type.invalid:
                        break;
                    case GuillotineRect.Type.singleRect:
                        int cont_type=0;
                        if (containable(guillotineCell.r1, image.width, image.height))
                            cont_type = 0b001;
                        else if (autoroll && containable(guillotineCell.r1, image.height, image.width))
                            cont_type = 0b011;
                        if (cont_type == 0)  // 入らない
                            continue;
                        // 内包する
                        image.x = guillotineCell.r1.x;
                        image.y = guillotineCell.r1.y;
                        image.valid = true;
                        if (cont_type & 0b010) // 回転
                            image.roll ^= true;
                        guillotineCell.type = GuillotineRect.Type.invalid;
                        auto newvr = guillotineCut(guillotineCell.r1, image.width, image.height);
                        newvr.minimize();
                        if (newvr.type != GuillotineRect.Type.invalid)
                            rects ~= newvr;
                        validationcounter++;
                        break;
                        
                    case GuillotineRect.Type.virtualPair:
                        int cont_type=0;
                        if (containable(guillotineCell.r1, image.width, image.height))
                            cont_type = 0b001;
                        else if (autoroll && containable(guillotineCell.r1, image.height, image.width))
                            cont_type = 0b011;
                        else if (containable(guillotineCell.r2, image.width, image.height))
                            cont_type = 0b101;
                        else if (autoroll && containable(guillotineCell.r2, image.height, image.width))
                            cont_type = 0b111;
                        if (cont_type == 0) // 入らない
                            continue;
                        // 入る
                        if (cont_type & 0b010) // 回転
                            image.roll ^= true;
                        // 備考：r1が縦長　r2が横長
                        if ((cont_type & 0b100) == 0) { // 1番目を使う
                            image.x = guillotineCell.r1.x;
                            image.y = guillotineCell.r1.y;
                            image.valid = true;
                            // カットして追加
                            auto newvr = guillotineCut(guillotineCell.r1, image.width, image.height);
                            // 再利用
                            auto remainder = GuillotineRect(GuillotineRect.Type.singleRect, guillotineCell.r2);
                            remainder.r1.width -= guillotineCell.r1.width;
                            guillotineCell.type = GuillotineRect.Type.invalid;
                            newvr.minimize();
                            remainder.minimize();
                            if (newvr.type != GuillotineRect.Type.invalid)
                                rects ~= newvr;
                            if (remainder.type != GuillotineRect.Type.invalid)
                                rects ~= remainder;
                        }
                        else { // 2番目を使う
                            image.x = guillotineCell.r2.x;
                            image.y = guillotineCell.r2.y;
                            image.valid = true;
                            // カットして追加
                            auto newvr = guillotineCut(guillotineCell.r2, image.width, image.height);
                            // 再利用
                            auto remainder = GuillotineRect(GuillotineRect.Type.singleRect, guillotineCell.r1);
                            remainder.r1.height -= guillotineCell.r1.height;
                            guillotineCell.type = GuillotineRect.Type.invalid;
                            newvr.minimize();
                            remainder.minimize();
                            if (newvr.type != GuillotineRect.Type.invalid)
                                rects ~= newvr;
                            if (remainder.type != GuillotineRect.Type.invalid)
                                rects ~= remainder;
                        }
                        validationcounter++;
                        break;
                    }
                }
            }
            
            if (validationcounter == 0)
                break;
        }

        foreach (ref image; img.assigns) {
            if (!image.valid) {
                foreach (ref image2; img.assigns) {
                    image2.valid = false;
                }
                return false;
            }
        }
        
        return true;
    }
}

