module texpzl.main;

import texpzl.texpzl;
import texpzl.shelfmap;
import texpzl.guillotinemap;
import std.file, std.stdio, std.path, std.exception;

bool verbosemode=false;

void compressImagesInFolder(string foldername, int padding, bool autoroll) {
    Texpzl_init(); // SDL_Imageを使うので初期化がいる
    scope(exit) Texpzl_quit();

    Images images;
    
    // 画像読み込み
    writef("Loading images .. ");
    stdout.flush();
    if (verbosemode)
        writeln();
    int imagescount;
    foreach (file; dirEntries(foldername, SpanMode.shallow)) {
        if (file.extension == ".png" ||
            file.extension == ".bmp") {
            auto abs_file = absolutePath(file);
            auto abs_folder = absolutePath(foldername);
            auto name = relativePath(abs_file, abs_folder);
            if (verbosemode)
                writeln("Load ", name);
            images.addImage(file, name, padding);
            imagescount++;
        }
    }
    writeln("Number: ", imagescount);

    // Atlas作成  アルゴリズム選択
    writef("Making atlas .. ");
    stdout.flush();
    //auto width = ShelfMap.doMapping(images, autoroll);
    auto width = GuillotineMap.doMapping(images, autoroll);

    // 密度計算してみる
    int maxdensity=width*width, density=0;
    foreach (a; images.assigns) {
        enforce(a.valid);
        density += a.width * a.height;
    }
    writefln("Complete: WIDTH %d, DENSITY %5.2f%%", width, cast(real)density / maxdensity * 100.0);

    // 画像作成
    writef("Making image .. ");
    stdout.flush();
    PixelsRGBA pixels;
    pixels.resize(width, width);
    foreach (a; images.assigns)
        doAssign(pixels, a); // 配置
    writef("compress .. ");
    stdout.flush();
    pixels.compress();
    writefln("Complete: %.2f KB", pixels.compressed_data.length/1024.0);

    // PNG書き込み
    writeln("Saving PNG ..");
    auto pngfile = foldername.baseName() ~ "_atlas.png";
    PNG.save(pixels, pngfile);
    writeln("Complete: ", pngfile);
 
    // 配置情報保存
    auto assignfile = foldername.baseName() ~ "_assign.csv";
    writeln("     CSV: ", assignfile);
    saveAssigns(assignfile, width, width, images.assigns, false);
    auto assignfile2 = foldername.baseName() ~ "_assign.texp";
    writeln("     BIN: ", assignfile2);
    saveAssigns(assignfile2, width, width, images.assigns, true);
    
}

import std.getopt;
import std.windows.charset;
import std.c.stdio : puts;
void main(string[] args) {
    bool noroll=false;
    int padding=1;
    getopt(args,
           "n",  &noroll,
           "p", &padding,
           "v", &verbosemode);
    if (args.length <= 1) {
        auto buf = cast(char[])read("Usage.txt");
        buf ~= 0;
        std.c.stdio.puts(buf.ptr);
    }
    else {
        compressImagesInFolder(args[1], padding, !noroll);
    }
}
